'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var core = require('@react-pdf-viewer/core');

var NextIcon = function () {
    return (React.createElement(core.Icon, { size: 16 },
        React.createElement("path", { d: "M0.541,5.627L11.666,18.2c0.183,0.207,0.499,0.226,0.706,0.043c0.015-0.014,0.03-0.028,0.043-0.043\n                L23.541,5.627" })));
};

var PreviousIcon = function () {
    return (React.createElement(core.Icon, { size: 16 },
        React.createElement("path", { d: "M23.535,18.373L12.409,5.8c-0.183-0.207-0.499-0.226-0.706-0.043C11.688,5.77,11.674,5.785,11.66,5.8\n                L0.535,18.373" })));
};

var SearchIcon = function () {
    return (React.createElement(core.Icon, { size: 16 },
        React.createElement("path", { d: "M10.5,0.5c5.523,0,10,4.477,10,10s-4.477,10-10,10s-10-4.477-10-10S4.977,0.5,10.5,0.5z\n                M23.5,23.5\n                l-5.929-5.929" })));
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var EMPTY_KEYWORD_REGEXP = new RegExp(' ');

var useDocument = function (store) {
    var _a = React.useState(store.get('doc')), currentDoc = _a[0], setCurrentDoc = _a[1];
    var handleDocumentChanged = function (doc) {
        setCurrentDoc(doc);
    };
    React.useEffect(function () {
        store.subscribe('doc', handleDocumentChanged);
        return function () {
            store.unsubscribe('doc', handleDocumentChanged);
        };
    }, []);
    return { currentDoc: currentDoc };
};

var useSearch = function (doc, store) {
    var _a = React.useState(''), keyword = _a[0], setKeyword = _a[1];
    var _b = React.useState([]), found = _b[0], setFound = _b[1];
    var _c = React.useState(0), currentMatch = _c[0], setCurrentMatch = _c[1];
    var _d = React.useState(false), matchCase = _d[0], setMatchCase = _d[1];
    var textContents = React.useRef([]);
    var _e = React.useState(false), wholeWords = _e[0], setWholeWords = _e[1];
    var indexArr = Array(doc.numPages)
        .fill(0)
        .map(function (_, i) { return i; });
    var changeMatchCase = function (isChecked) {
        setMatchCase(isChecked);
        if (keyword) {
            searchFor(keyword, isChecked, wholeWords);
        }
    };
    var changeWholeWords = function (isChecked) {
        setWholeWords(isChecked);
        if (keyword) {
            searchFor(keyword, matchCase, isChecked);
        }
    };
    var jumpToPreviousMatch = function () {
        if (!keyword) {
            return;
        }
        var prev = currentMatch - 1;
        var updated = prev > 0 ? prev : found.length;
        setCurrentMatch(updated);
        jumpToMatch(found[updated - 1]);
    };
    var jumpToNextMatch = function () {
        if (!keyword) {
            return;
        }
        var next = currentMatch + 1;
        var updated = next <= found.length ? next : 1;
        setCurrentMatch(updated);
        jumpToMatch(found[updated - 1]);
    };
    var clearKeyword = function () {
        if (!keyword) {
            return;
        }
        store.update('keyword', [EMPTY_KEYWORD_REGEXP]);
        setKeyword('');
        setCurrentMatch(0);
        setFound([]);
        setMatchCase(false);
        setWholeWords(false);
    };
    var search = function () { return searchFor(keyword, matchCase, wholeWords); };
    var buildKeywordRegex = function (keywordParam, matchCaseParam, wholeWordsParam) {
        var source = wholeWordsParam ? " " + keywordParam + " " : keywordParam;
        var flags = matchCaseParam ? 'g' : 'gi';
        return new RegExp(source, flags);
    };
    var getTextContents = function () {
        var promises = indexArr.map(function (pageIndex) {
            return doc
                .getPage(pageIndex + 1)
                .then(function (page) {
                return page.getTextContent();
            })
                .then(function (content) {
                var pageContent = content.items
                    .map(function (item) { return item.str || ''; })
                    .join('');
                return Promise.resolve({
                    pageContent: pageContent,
                    pageIndex: pageIndex,
                });
            });
        });
        return Promise.all(promises).then(function (data) {
            data.sort(function (a, b) { return a.pageIndex - b.pageIndex; });
            return Promise.resolve(data.map(function (item) { return item.pageContent; }));
        });
    };
    var jumpToMatch = function (match) {
        var jumpToPage = store.get('jumpToPage');
        if (jumpToPage) {
            jumpToPage(match.pageIndex);
        }
        store.update('match', match);
    };
    var searchFor = function (keywordParam, matchCaseParam, wholeWordsParam) {
        var regexp = buildKeywordRegex(keywordParam, matchCaseParam, wholeWordsParam);
        store.update('keyword', [regexp]);
        setCurrentMatch(0);
        setFound([]);
        var promise = textContents.current.length === 0
            ? getTextContents().then(function (response) {
                textContents.current = response;
                return Promise.resolve(response);
            })
            : Promise.resolve(textContents.current);
        promise.then(function (response) {
            var arr = [];
            response.forEach(function (item, pageIndex) {
                var numMatches = (item.match(regexp) || []).length;
                for (var matchIndex = 0; matchIndex < numMatches; matchIndex++) {
                    arr.push({
                        matchIndex: matchIndex,
                        pageIndex: pageIndex,
                    });
                }
            });
            setFound(arr);
            if (arr.length > 0) {
                setCurrentMatch(1);
                jumpToMatch(arr[0]);
            }
        });
    };
    return {
        clearKeyword: clearKeyword,
        changeMatchCase: changeMatchCase,
        changeWholeWords: changeWholeWords,
        currentMatch: currentMatch,
        jumpToNextMatch: jumpToNextMatch,
        jumpToPreviousMatch: jumpToPreviousMatch,
        keyword: keyword,
        matchCase: matchCase,
        numberOfMatches: found.length,
        wholeWords: wholeWords,
        search: search,
        setKeyword: setKeyword,
    };
};

var Inner = function (_a) {
    var children = _a.children, doc = _a.doc, store = _a.store;
    var result = useSearch(doc, store);
    return children(__assign({}, result));
};
var Search = function (_a) {
    var children = _a.children, store = _a.store;
    var currentDoc = useDocument(store).currentDoc;
    return (currentDoc
        ? React.createElement(Inner, { doc: currentDoc, store: store }, children)
        : React.createElement(React.Fragment, null));
};

var PORTAL_OFFSET = { left: 0, top: 8 };
var SearchPopover = function (_a) {
    var doc = _a.doc, store = _a.store, onToggle = _a.onToggle;
    var l10n = React.useContext(core.LocalizationContext);
    var _b = useSearch(doc, store), clearKeyword = _b.clearKeyword, changeMatchCase = _b.changeMatchCase, changeWholeWords = _b.changeWholeWords, currentMatch = _b.currentMatch, jumpToNextMatch = _b.jumpToNextMatch, jumpToPreviousMatch = _b.jumpToPreviousMatch, keyword = _b.keyword, matchCase = _b.matchCase, numberOfMatches = _b.numberOfMatches, wholeWords = _b.wholeWords, search = _b.search, setKeyword = _b.setKeyword;
    var onChangeKeyword = function (e) {
        setKeyword(e.target.value);
    };
    var onKeydownSearch = function (e) {
        if (e.keyCode !== 13 || !keyword) {
            return;
        }
        search();
    };
    var onChangeMatchCase = function (e) {
        changeMatchCase(e.target.checked);
    };
    var onChangeWholeWords = function (e) {
        changeWholeWords(e.target.checked);
    };
    var onClose = function () {
        onToggle();
        clearKeyword();
    };
    return (React.createElement("div", { className: 'rpv-search-popover' },
        React.createElement("div", { className: 'rpv-search-popover-input-counter' },
            React.createElement("input", { className: 'rpv-search-popover-input', placeholder: (l10n && l10n.search ? l10n.search.enterToSearch : 'Enter to search'), type: "text", value: keyword, onChange: onChangeKeyword, onKeyDown: onKeydownSearch }),
            React.createElement("div", { className: 'rpv-search-popover-counter' },
                currentMatch,
                "/",
                numberOfMatches)),
        React.createElement("label", { className: 'rpv-search-popover-label' },
            React.createElement("input", { className: 'rpv-search-popover-label-checkbox', checked: matchCase, type: "checkbox", onChange: onChangeMatchCase }),
            " ",
            l10n && l10n.search ? l10n.search.matchCase : 'Match case'),
        React.createElement("label", { className: 'rpv-search-popover-label' },
            React.createElement("input", { className: 'rpv-search-popover-label-checkbox', checked: wholeWords, type: "checkbox", onChange: onChangeWholeWords }),
            " ",
            l10n && l10n.search ? l10n.search.wholeWords : 'Whole words'),
        React.createElement("div", { className: 'rpv-search-popover-footer' },
            React.createElement("div", { className: 'rpv-search-popover-footer-item' },
                React.createElement(core.Tooltip, { position: core.Position.BottomCenter, target: React.createElement(core.Button, { onClick: jumpToPreviousMatch },
                        React.createElement(PreviousIcon, null)), content: function () { return (l10n && l10n.search ? l10n.search.previousMatch : 'Previous match'); }, offset: PORTAL_OFFSET })),
            React.createElement("div", { className: 'rpv-search-popover-footer-item' },
                React.createElement(core.Tooltip, { position: core.Position.BottomCenter, target: React.createElement(core.Button, { onClick: jumpToNextMatch },
                        React.createElement(NextIcon, null)), content: function () { return (l10n && l10n.search ? l10n.search.nextMatch : 'Next match'); }, offset: PORTAL_OFFSET })),
            React.createElement("div", { className: 'rpv-search-popover-footer-button' },
                React.createElement(core.PrimaryButton, { onClick: onClose }, l10n && l10n.search ? l10n.search.close : 'Close')))));
};

var ShowSearchPopoverDecorator = function (_a) {
    var children = _a.children, onClick = _a.onClick;
    var l10n = React.useContext(core.LocalizationContext);
    var label = (l10n && l10n.search ? l10n.search.search : 'Search');
    var icon = React.createElement(SearchIcon, null);
    return children({ icon: icon, label: label, onClick: onClick });
};

var TOOLTIP_OFFSET = { left: 0, top: 8 };
var ShowSearchPopoverButton = function (_a) {
    var onClick = _a.onClick;
    return (React.createElement(ShowSearchPopoverDecorator, { onClick: onClick }, function (p) { return (React.createElement(core.Tooltip, { position: core.Position.BottomCenter, target: React.createElement(core.Button, { onClick: onClick }, p.icon), content: function () { return p.label; }, offset: TOOLTIP_OFFSET })); }));
};

var PORTAL_OFFSET$1 = { left: 0, top: 8 };
var ShowSearchPopover = function (_a) {
    var children = _a.children, store = _a.store;
    var currentDoc = useDocument(store).currentDoc;
    var defaultChildren = function (props) { return (React.createElement(ShowSearchPopoverButton, __assign({}, props))); };
    var render = children || defaultChildren;
    return (currentDoc
        ? (React.createElement(core.Popover, { position: core.Position.BottomLeft, target: function (toggle) { return render({
                onClick: toggle,
            }); }, content: function (toggle) { return React.createElement(SearchPopover, { doc: currentDoc, store: store, onToggle: toggle }); }, offset: PORTAL_OFFSET$1, closeOnClickOutside: false, closeOnEscape: true }))
        : React.createElement(React.Fragment, null));
};

var calculateOffset = function (children, parent) {
    var top = children.offsetTop;
    var left = children.offsetLeft;
    var p = children.parentElement;
    while (p && p !== parent) {
        top += p.offsetTop;
        left += p.offsetLeft;
        p = p.parentElement;
    }
    return {
        left: left,
        top: top,
    };
};

var removeNode = function (ele) {
    var parent = ele.parentNode;
    if (parent) {
        parent.removeChild(ele);
    }
};
var replaceNode = function (replacementNode, node) {
    removeNode(replacementNode);
    var parent = node.parentNode;
    if (parent) {
        parent.insertBefore(replacementNode, node);
    }
    removeNode(node);
};
var unwrap = function (ele) {
    var parent = ele.parentNode;
    if (!parent) {
        return;
    }
    var range = document.createRange();
    range.selectNodeContents(ele);
    replaceNode(range.extractContents(), ele);
    parent.normalize();
};

var Tracker = function (_a) {
    var pageIndex = _a.pageIndex, store = _a.store;
    var _b = React.useState({
        matchIndex: -1,
        pageIndex: -1,
    }), match = _b[0], setMatch = _b[1];
    var _c = React.useState([EMPTY_KEYWORD_REGEXP]), keywordRegexp = _c[0], setKeywordRegexp = _c[1];
    var _d = React.useState({
        pageIndex: pageIndex,
        scale: 1,
        status: core.LayerRenderStatus.PreRender,
    }), renderStatus = _d[0], setRenderStatus = _d[1];
    var currentMatchRef = React.useRef(null);
    var unhighlightAll = function (containerEle) {
        var highlightNodes = containerEle.querySelectorAll('span.rpv-search-text-highlight');
        var total = highlightNodes.length;
        for (var i = 0; i < total; i++) {
            highlightNodes[i].parentElement.removeChild(highlightNodes[i]);
        }
    };
    var highlight = function (keyword, containerEle, span) {
        var text = span.textContent;
        if (!keyword.source.trim() || !text) {
            return;
        }
        var startOffset = text.search(keyword);
        var firstChild = span.firstChild;
        if (startOffset === -1 || !firstChild) {
            return;
        }
        var endOffset = startOffset + keyword.source.length;
        var range = document.createRange();
        range.setStart(firstChild, startOffset);
        range.setEnd(firstChild, endOffset);
        var wrapper = document.createElement('span');
        range.surroundContents(wrapper);
        var wrapperRect = wrapper.getBoundingClientRect();
        var containerRect = containerEle.getBoundingClientRect();
        var highlightEle = document.createElement('span');
        containerEle.appendChild(highlightEle);
        highlightEle.style.left = 100 * (wrapperRect.left - containerRect.left) / containerRect.width + "%";
        highlightEle.style.top = 100 * (wrapperRect.top - containerRect.top) / containerRect.height + "%";
        highlightEle.style.width = 100 * (wrapperRect.width) / containerRect.width + "%";
        highlightEle.style.height = 100 * (wrapperRect.height) / containerRect.height + "%";
        highlightEle.classList.add('rpv-search-text-highlight');
        unwrap(wrapper);
    };
    var highlightAll = function (containerEle) {
        var spans = [].slice.call(containerEle.querySelectorAll('.rpv-core-text'));
        keywordRegexp.forEach(function (keyword) {
            spans.forEach(function (span) {
                highlight(keyword, containerEle, span);
            });
        });
    };
    var handleKeywordChanged = function (keyword) {
        if (keyword && keyword.length > 0) {
            setKeywordRegexp(keyword);
        }
    };
    var handleMatchChanged = function (currentMatch) {
        setMatch(currentMatch);
    };
    var handleRenderStatusChanged = function (status) {
        if (!status.has(pageIndex)) {
            return;
        }
        var currentStatus = status.get(pageIndex);
        if (currentStatus) {
            setRenderStatus({
                ele: currentStatus.ele,
                pageIndex: pageIndex,
                scale: currentStatus.scale,
                status: currentStatus.status,
            });
        }
    };
    var isEmptyKeyword = function () { return keywordRegexp.length === 0 || (keywordRegexp.length === 1 && keywordRegexp[0].source.trim() === ''); };
    React.useEffect(function () {
        if (isEmptyKeyword() || !renderStatus.ele || renderStatus.status !== core.LayerRenderStatus.DidRender) {
            return;
        }
        var containerEle = renderStatus.ele;
        unhighlightAll(containerEle);
        highlightAll(containerEle);
        scrollToMatch();
    }, [keywordRegexp, match, renderStatus.status]);
    React.useEffect(function () {
        if (isEmptyKeyword() && renderStatus.ele && renderStatus.status === core.LayerRenderStatus.DidRender) {
            unhighlightAll(renderStatus.ele);
        }
    }, [keywordRegexp, renderStatus.status]);
    var scrollToMatch = function () {
        if (match.pageIndex !== pageIndex || !renderStatus.ele || renderStatus.status !== core.LayerRenderStatus.DidRender) {
            return;
        }
        var container = renderStatus.ele;
        var spans = container.querySelectorAll('.rpv-search-text-highlight');
        if (match.matchIndex < spans.length) {
            var span = spans[match.matchIndex];
            var _a = calculateOffset(span, container), left = _a.left, top_1 = _a.top;
            var jump = store.get('jumpToDestination');
            if (jump) {
                jump(pageIndex, (container.getBoundingClientRect().height - top_1) / renderStatus.scale, left / renderStatus.scale, renderStatus.scale);
                if (currentMatchRef.current) {
                    currentMatchRef.current.classList.remove('rpv-search-text-highlight-current');
                }
                currentMatchRef.current = span;
                span.classList.add('rpv-search-text-highlight-current');
            }
        }
    };
    React.useEffect(function () {
        store.subscribe('keyword', handleKeywordChanged);
        store.subscribe('match', handleMatchChanged);
        store.subscribe('renderStatus', handleRenderStatusChanged);
        return function () {
            store.unsubscribe('keyword', handleKeywordChanged);
            store.unsubscribe('match', handleMatchChanged);
            store.unsubscribe('renderStatus', handleRenderStatusChanged);
        };
    }, []);
    return React.createElement(React.Fragment, null);
};

var searchPlugin = function (props) {
    var store = React.useMemo(function () { return core.createStore({
        renderStatus: new Map(),
    }); }, []);
    var SearchDecorator = function (props) { return (React.createElement(Search, __assign({}, props, { store: store }))); };
    var ShowSearchPopoverDecorator = function (props) { return (React.createElement(ShowSearchPopover, __assign({}, props, { store: store }))); };
    var ShowSearchPopoverButtonDecorator = function () { return (React.createElement(ShowSearchPopoverDecorator, null, function (props) { return React.createElement(ShowSearchPopoverButton, __assign({}, props)); })); };
    var renderViewer = function (props) {
        var currentSlot = props.slot;
        if (currentSlot.subSlot) {
            currentSlot.subSlot.children = (React.createElement(React.Fragment, null,
                Array(props.doc.numPages).fill(0).map(function (_, index) { return (React.createElement(Tracker, { key: index, pageIndex: index, store: store })); }),
                currentSlot.subSlot.children));
        }
        return currentSlot;
    };
    var normalizeKeywordItem = function (keyword) {
        return typeof keyword === 'string'
            ? (keyword === '' ? EMPTY_KEYWORD_REGEXP : new RegExp(keyword))
            : (keyword || EMPTY_KEYWORD_REGEXP);
    };
    return {
        install: function (pluginFunctions) {
            var keyword = [EMPTY_KEYWORD_REGEXP];
            if (props) {
                if (Array.isArray(props.keyword)) {
                    keyword = (props.keyword).map(function (k) { return normalizeKeywordItem(k); });
                }
                else {
                    keyword = [normalizeKeywordItem(props.keyword)];
                }
            }
            store.update('jumpToDestination', pluginFunctions.jumpToDestination);
            store.update('jumpToPage', pluginFunctions.jumpToPage);
            store.update('keyword', keyword);
        },
        renderViewer: renderViewer,
        uninstall: function (props) {
            var renderStatus = store.get('renderStatus');
            if (renderStatus) {
                renderStatus.clear();
            }
        },
        onDocumentLoad: function (props) {
            store.update('doc', props.doc);
        },
        onTextLayerRender: function (props) {
            var renderStatus = store.get('renderStatus');
            if (renderStatus) {
                renderStatus = renderStatus.set(props.pageIndex, props);
                store.update('renderStatus', renderStatus);
            }
        },
        Search: SearchDecorator,
        ShowSearchPopover: ShowSearchPopoverDecorator,
        ShowSearchPopoverButton: ShowSearchPopoverButtonDecorator,
    };
};

exports.NextIcon = NextIcon;
exports.PreviousIcon = PreviousIcon;
exports.SearchIcon = SearchIcon;
exports.searchPlugin = searchPlugin;
